#####


##======================= load and process the data 


# load IHC data:
ihc = read.csv("IHCScoring for R.csv",stringsAsFactors = FALSE)
# put on log scale:
ihc[,3]=as.numeric(ihc[,3])
ihc[,-(1:2)] = log2(ihc[,-(1:2)])
# replace -Inf w 0:
for(i in 3:ncol(ihc)){ihc[,i]=pmax(ihc[,i],0)}
# recalculate averages:
ihc$CD3 = rowMeans(ihc[,paste0("CD3.ROI.",1:5)],na.rm=T)
ihc$CD8 = rowMeans(ihc[,paste0("CD8.ROI.",1:5)],na.rm=T)

# add variables for number of ROI = NA
ihc$NA.ROI = rowSums(is.na(ihc[,paste0("CD3.ROI.",1:5)]))

# load the nanostring cell types data:
#cells = read.csv("cell type scores - raw.csv",stringsAsFactors = FALSE)

# load the nanostring normalized data:
nanostringdata = read.csv("raw nanostring data for IHC comparison.csv",stringsAsFactors = F,row.names = 1)
nanostringdata = nanostringdata[!is.na(nanostringdata[,"A2M"]),]
raw = nanostringdata[,-(1:3)]

# some lanes had very low expressoin and should probably be discarded. Use the HK factors to do so. 
# load HK gene list:
hks = read.csv("selected housekeeping genes.csv",stringsAsFactors = FALSE,header=T)[,1]
#hks = hks[hks[,2]!="discarded",1]
#hks = make.names(hks)
# load the raw data and get the hk geomeans:
hkg = apply(log2(raw[,hks]),1,mean)
hist(hkg,breaks=10)
plot(hkg[order(hkg)])
abline(h=6.5)
fail = hkg<8
if(!identical(rownames(nanostringdata),rownames(raw))){stop("data misalignment")}

# also fail all but the highest hkg duplicate of each ID:
ids = unique(nanostringdata$ID)
for(id in ids)
{
  use = (nanostringdata$ID==id)
  if(sum(use)>1)
  {
    drop = hkg[use]!=max(hkg[use])
    fail[use][drop]=TRUE  
  }
}

# normalize the raw data:
norm = sweep(log2(raw),1,hkg,"-")

# calculate cell type scores:
### calc cell types from raw:
# load gene annotation:
gannot = read.csv("gannot - cip - junecelltype.csv",row.names=1)
sharedgenes = intersect(colnames(norm),rownames(gannot))
gannot = gannot[sharedgenes,]

all.cells = setdiff(unique(gannot$Cell.Type.TCGA2),c("",NA))
cells = matrix(NA,nrow(norm),length(all.cells)); colnames(cells)=all.cells
for(i in 1:length(all.cells))
{
  tempgenes = (gannot$Cell.Type.TCGA2==all.cells[i])&(!is.na(gannot$Cell.Type.TCGA2))
  cells[,i] = apply(norm[,tempgenes,drop=FALSE],1,mean)
}

# remove low hkg lanes:
cells = cells[!fail,]
cells = as.data.frame(cells)
cells$hkg = hkg[!fail]
cells$ID = nanostringdata$ID[!fail]

## align rows of cells to rows of ihc:
rownames(ihc)=make.names(ihc$PhenoPath.ID)
rownames(cells)=make.names(cells$ID)
colnames(cells) = make.names(colnames(cells))
sharedids = intersect(rownames(ihc),rownames(cells))
sharedids
setdiff(rownames(ihc),rownames(cells))
setdiff(rownames(cells),rownames(ihc))

setdiff(make.names(ihc$PhenoPath.ID),make.names(cells$ID))

ihc[setdiff(rownames(ihc),rownames(cells)),1:5]

cells = cells[sharedids,]
ihc = ihc[sharedids,]

# remove poor samples (<3 ROI):
ihc[ihc$NA.ROI>=2,]=NA


##======================= analyze:
cells$cd8cd3 = (cells$CD8.T.cells-cells$T.cells)
ihc$cd8cd3 = (ihc$CD8-ihc$CD3)

##### get cell scores after adjusting for HKG:
plot(cells$T.cells~cells$hkg)
# get cell types residuals from hkg matrix:
cells.r = replace(cells,TRUE,NA)
for(i in which(!is.element(colnames(cells),c("hkg","ID","cd8cd3"))))
{
  mod = lm(cells[,i]~cells$hkg)
  cells.r[,i] = mod$resid
}
pairs(cbind(cells$T.cells,cells.r$T.cells,cells$hkg,ihc$CD3),labels = c("Tcells raw","Tcells resid","HKG","IHC CD3"))



pdf("IHC vs expression for paper.pdf",height=4)
par(mfrow=c(1,2))
# cd3:
plot(cells$T.cells,ihc$CD3,pch=16,ylim=range(ihc[,paste0("CD3.ROI.",1:5)],na.rm=T),xlab="Whole FFPE gene expression",ylab="IHC from tumor interior",main="T-cells") 
abline(mean(ihc$CD3,na.rm=T)-mean(cells$T.cells),1)
abline(lm(ihc$CD3~cells$T.cells),lty=2)
legend("bottomright",cex=.75,
       legend=c(paste("cor =",round(cor(cells$T.cells,ihc$CD3,use="complete"),3)),
                paste("RMSE =",round(sqrt(mean((cells$T.cells-ihc$CD3-mean(cells$T.cells-ihc$CD3,na.rm=T))^2,na.rm=T)),3)),
                paste("p =",round(summary(lm(cells$T.cells~ihc$CD3))$coef[2,4],3))))


plot(cells$CD8.T.cells,ihc$CD8,pch=16,ylim=range(ihc[,paste0("CD8.ROI.",1:5)],na.rm=T),xlab="Whole FFPE gene expression",ylab="IHC from tumor interior",main="CD8 T-cells") 
abline(mean(pmax(0,ihc$CD8),na.rm=T)-mean(cells$CD8.T.cells),1)
abline(lm(ihc$CD8~cells$CD8.T.cells),lty=2)
legend("bottomright",cex=.75,
       legend=c(paste("cor =",round(cor(cells$CD8.T.cells,ihc$CD8,use="complete"),3)),
                paste("RMSE =",round(sqrt(mean((cells$CD8.T.cells-ihc$CD8-mean(cells$CD8.T.cells-ihc$CD8,na.rm=T))^2,na.rm=T)),3)),
                paste("p =",round(summary(lm(cells$CD8.T.cells~ihc$CD8))$coef[2,4],3))))
dev.off()



pdf("IHC vs expression.pdf")
# cd3:
plot(cells$T.cells,ihc$CD3,pch=16,ylim=range(ihc[,paste0("CD3.ROI.",1:5)],na.rm=T),xlab="Whole FFPE gene expression",ylab="IHC from tumor interior",main="T-cells") 
abline(mean(ihc$CD3,na.rm=T)-mean(cells$T.cells),1)
abline(lm(ihc$CD3~cells$T.cells),lty=2)
legend("bottomright",pch=c(NA,NA,NA,NA,NA),lty=c(1,2,NA,NA,NA),
       legend=c("Unit slope","Best fit",paste("cor =",round(cor(cells$T.cells,ihc$CD3,use="complete"),3)),
                                        paste("RMSE =",round(sqrt(mean((cells$T.cells-ihc$CD3-mean(cells$T.cells-ihc$CD3,na.rm=T))^2,na.rm=T)),3)),
                                        paste("p =",round(summary(lm(cells$T.cells~ihc$CD3))$coef[2,4],3))))

# resids:
plot(cells.r$T.cells,ihc$CD3,pch=16,ylim=range(ihc[,paste0("CD3.ROI.",1:5)],na.rm=T),
     xlab="Whole FFPE gene expression",ylab="IHC from tumor interior",main="T-cells (residuals)") 
abline(mean(ihc$CD3,na.rm=T)-mean(cells.r$T.cells),1)
abline(lm(ihc$CD3~cells.r$T.cells),lty=2)
legend("bottomright",pch=c(NA,NA,NA,NA),lty=c(1,2,NA,NA),
       legend=c("Unit slope","Best fit",paste("cor =",round(cor(cells.r$T.cells,ihc$CD3,use="complete"),3)),
                paste("p =",round(summary(lm(cells.r$T.cells~ihc$CD3))$coef[2,4],3))))


plot(cells$T.cells,ihc$CD3,pch=16,ylim=range(ihc[,paste0("CD3.ROI.",1:5)],na.rm=T),xlab="Whole FFPE gene expression",ylab="IHC from tumor interior",main="T-cells") 
abline(mean(ihc$CD3,na.rm=T)-mean(cells$T.cells),1)
abline(lm(ihc$CD3~cells$T.cells),lty=2)
for(i in 1:5)
{
  points(cells$T.cells,ihc[,paste0("CD3.ROI.",i)],col=2)
}
for(i in 1:nrow(cells))
{
  lines(rep(cells$T.cells[i],2),range(ihc[i,paste0("CD3.ROI.",1:5)],na.rm=T),col=2)
}
legend("bottomright",pch=c(16,1,NA,NA,NA,NA),lty=c(NA,NA,1,2,NA,NA),col=c(1,2,1,1),
       legend=c("Mean IHC","Single ROI","Unit slope","Best fit",paste("cor =",round(cor(cells$T.cells,ihc$CD3,use="complete"),3)),
                paste("p =",round(summary(lm(cells$T.cells~ihc$CD3))$coef[2,4],3))))

# PR13.126.13B is the outlier

# cd8:

plot(cells$CD8.T.cells,ihc$CD8,pch=16,ylim=range(ihc[,paste0("CD8.ROI.",1:5)],na.rm=T),xlab="Whole FFPE gene expression",ylab="IHC from tumor interior",main="CD8 T-cells") 
abline(mean(pmax(0,ihc$CD8),na.rm=T)-mean(cells$CD8.T.cells),1)
abline(lm(ihc$CD8~cells$CD8.T.cells),lty=2)
legend("bottomright",pch=c(NA,NA,NA,NA),lty=c(1,2,NA,NA),
       legend=c("Unit slope","Best fit",paste("cor =",round(cor(cells$CD8.T.cells,ihc$CD8,use="complete"),3)),
                paste("RMSE =",round(sqrt(mean((cells$CD8.T.cells-ihc$CD8-mean(cells$CD8.T.cells-ihc$CD8,na.rm=T))^2,na.rm=T)),3)),
                paste("p =",round(summary(lm(cells$CD8.T.cells~ihc$CD8))$coef[2,4],3))))

# resids:
plot(cells.r$CD8.T.cells,ihc$CD8,pch=16,ylim=range(ihc[,paste0("CD8.ROI.",1:5)],na.rm=T),
     xlab="Whole FFPE gene expression",ylab="IHC from tumor interior",main="CD8 T-cells (residuals)") 
abline(mean(pmax(0,ihc$CD8),na.rm=T)-mean(cells.r$CD8.T.cells),1)
abline(lm(ihc$CD8~cells.r$CD8.T.cells),lty=2)
legend("bottomright",pch=c(NA,NA,NA,NA),lty=c(1,2,NA,NA),
       legend=c("Unit slope","Best fit",paste("cor =",round(cor(cells.r$CD8.T.cells,ihc$CD8,use="complete"),3)),
                paste("p =",round(summary(lm(cells.r$CD8.T.cells~ihc$CD8))$coef[2,4],3))))



plot(cells$CD8.T.cells,ihc$CD8,pch=16,ylim=range(ihc[,paste0("CD8.ROI.",1:5)],na.rm=T),xlab="Whole FFPE gene expression",ylab="IHC from tumor interior",main="CD8 T-cells") 
abline(mean(pmax(0,ihc$CD8),na.rm=T)-mean(cells$CD8.T.cells),1)
abline(lm(ihc$CD8~cells$CD8.T.cells),lty=2)
for(i in 1:5)
{
  points(cells$CD8.T.cells,ihc[,paste0("CD8.ROI.",i)],col=2)
}
for(i in 1:nrow(cells))
{
  lines(rep(cells$CD8.T.cells[i],2),range(ihc[i,paste0("CD8.ROI.",1:5)],na.rm=T),col=2)
}
legend("bottomright",pch=c(16,1,NA,NA,NA,NA),lty=c(NA,NA,1,2,NA,NA),col=c(1,2,1,1),
       legend=c("Mean IHC","Single ROI","Unit slope","Best fit",paste("cor =",round(cor(cells$CD8.T.cells,ihc$CD8,use="complete"),3)),
                paste("p =",round(summary(lm(cells$CD8.T.cells~ihc$CD8))$coef[2,4],3))))
# PR13.126.13B is the outlier

# cd8/cd3:
plot(cells$cd8cd3,ihc$cd8cd3,pch=16,ylim=range(ihc[,paste0("CD8.ROI.",1:5)]-ihc[,paste0("CD3.ROI.",1:5)],na.rm=T),xlab="Whole FFPE gene expression",ylab="IHC from tumor interior",main="CD8/CD3 ratio") 
abline(mean(ihc$cd8cd3[ihc$cd8cd3>-Inf],na.rm=T)-mean(cells$cd8cd3),1)
abline(lm(ihc$cd8cd3~cells$cd8cd3),lty=2)
for(i in 1:5)
{
  points(cells$cd8cd3,ihc[,paste0("CD8.ROI.",i)]-ihc[,paste0("CD3.ROI.",i)],col=2)
}
for(i in 1:nrow(cells))
{
  lines(rep(cells$cd8cd3[i],2),range(ihc[i,paste0("CD8.ROI.",1:5)]-ihc[i,paste0("CD3.ROI.",1:5)],na.rm=T),col=2)
}
legend("bottomright",paste("cor =",round(cor(cells$cd8cd3,ihc$cd8cd3,use="complete"),3)))

dev.off()

